<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2021 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

require_once(dirname(__FILE__).'/../../classes/VatCheckerTable.php');

class AdminVatCheckerController extends ModuleAdminController
{
    public function __construct()
    {
        parent::__construct();
        $this->bootstrap = true;
        $this->meta_title = 'VatChecker';
        if (!$this->module->active) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminHome'));
        }
        $this->table = 'vatchecker';
        $this->identifier = 'id';
        $this->className = 'VatCheckerTable';
        $this->allow_export = true;
        $this->_defaultOrderBy = 'a.id';
        $this->_defaultOrderWay = 'DESC';
        $this->_default_pagination = 20;
        $this->_select = 'c.`firstname` as `firstname`, c.`lastname` as `lastname`, c.email as `email`, g.`name` as `cGroup`';
        $this->_join = '
            LEFT JOIN `' . _DB_PREFIX_ . 'customer` c ON (c.`id_customer` = a.`id_customer`)
            LEFT JOIN `' . _DB_PREFIX_ . 'group_lang` g ON (g.`id_group` = c.`id_default_group` AND g.`id_lang` = ' . $this->context->language->id . ')
        ';
        $this->addRowAction('edit');
        $this->addRowAction('delete');
        $this->bulk_actions = [
            'delete' => [
                'text' => $this->module->l('Delete selected'),
                'confirm' => $this->module->l('Delete selected items?')
            ]
        ];
        $this->list_no_link = true;
        $this->fields_list = [
            'id' => ['title' => 'ID', 'class' => 'fixed-width-xs', 'align' => 'center'],
            'firstname' => ['title' => $this->module->l('Firstname'), 'align' => 'center'],
            'lastname' => ['title' => $this->module->l('Lastname'), 'align' => 'center'],
            'cGroup' => [
                'title' => $this->module->l('Group'),
                'align' => 'center',
                'filter_key' => 'g!name'
            ],
            'email' => ['title' => $this->module->l('Email'), 'align' => 'center'],
            'active' => [
                'title' => $this->module->l('VAT-Free'),
                'align' => 'center',
                'active' => 'status',
                'type' => 'bool',
                'orderby' => false,
                'filter_key' => 'a!active',
                'class' => 'fixed-width-sm',
            ],
            'date_add' => [
                'title' => $this->module->l('Creation date'),
                'type'=>'datetime', 'align' => 'center',
                'filter_key'=>'a!date_add'
            ],
            'date_upd' => [
                'title' => $this->module->l('Update date'),
                'type'=>'datetime',
                'align' => 'center',
                'filter_key'=>'a!date_upd'
            ]
        ];
    }

    public function renderForm()
    {
        if ($this->action == 'new') {
            $this->object->period = 30;
        }
        if (!$this->object->id_customer) {
            $ps_customers = Customer::getCustomers(true);
            $table_customers = $this->getCustomersForceNoVat();
            foreach ($ps_customers as $k => $v) {
                $ps_customers[$k]['full_data'] = $v['firstname'] . ' ' . $v['lastname'] . ' (' . $v['email'] . ')';
                foreach ($table_customers as $t) {
                    if ($v['id_customer'] == $t) {
                        unset($ps_customers[$k]);
                    }
                }
            }
            array_unshift($ps_customers, ['id_customer' => 0, 'full_data' => '--- ' . $this->module->l('Please select customer') . ' ---']);
            $AddOrEdit = [
                'type' => 'select',
                'label' => $this->module->l('Customer'),
                'desc' => '',
                'name' => 'id_customer',
                'required' => true,
                'options' => [
                    'query' => $ps_customers,
                    'id' => 'id_customer',
                    'name' => 'full_data'
                ]
            ];
        } else {
            $ps_customer = new Customer($this->object->id_customer);
            $AddOrEdit = [
                'name' => 'info',
                'type' => 'html',
                'html_content' =>
                    '<div class="alert alert-warning">
                        ' . $this->module->l('Customer') . '
                        <strong>
                        ' . $ps_customer->firstname . '
                        ' . $ps_customer->lastname . '
                        (' . $ps_customer->email . ')
                        </strong>
                    </div>'
            ];
        }
        $this->fields_form = [
            'legend' => [
                'title' => $this->module->l('Unset VAT Force'),
                'icon' => 'icon-list-ul'
            ],
            'input' => [
                $AddOrEdit,
                [
                    'name' => 'active',
                    'type' => 'switch',
                    'label' => $this->module->l('Unset VAT'),
                    'desc' => 'Force unset VAT for this customer',
                    'values' => [
                        ['id' => 'active_on', 'value' => 1, 'label' => 'Yes'],
                        ['id' => 'active_off', 'value' => 0, 'label' => 'No']
                    ]
                ],
            ],
            'submit' => [
                'title' => $this->module->l('Save'),
            ]
        ];
        return parent::renderForm();
    }

    public function setMedia($var = false)
    {
        parent::setMedia($var);
        $this->context->controller->addJQueryPlugin('select2');
    }

    public function renderView()
    {
        $this->context->smarty->assign([
            'module_dir' => _MODULE_DIR_.$this->module->name.'/'
        ]);
        $tpl = $this->context->smarty->createTemplate('module:vatchecker/views/templates/admin/configure-table.tpl');
        return $tpl->fetch();
    }

    public function initContent()
    {
        $this->content = $this->renderView();
        return parent::initContent();
    }

    public function postProcess()
    {
        $this->reloadAPCu();
        return parent::postProcess();
    }

    private function getCustomersForceNoVat()
    {
        if ($query = Db::getInstance()->executeS(
            'SELECT `id_customer` FROM `' . _DB_PREFIX_ . $this->table . '`'
        )) {
            foreach ($query as $q) {
                $return[] = $q['id_customer'];
            }
        }
        return $return ?? [];
    }

    private function reloadAPCu()
    {
        if (function_exists('apcu_delete')) {
            apcu_delete('vatchecker');
        }
    }
}
