<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class VatChecker extends TaxManagerModule
{
    private $euVatCountriesAndFormats = [
        'AT'  => '(AT)?U[0-9]{8}',                                    # Austria
        'BE'  => '(BE)?0[0-9]{9}',                                    # Belgium
        'BG'  => '(BG)?[0-9]{9,10}',                                  # Bulgaria
        'CY'  => '(CY)?[0-9]{8}[A-Z]',                                # Cyprus
        'CZ'  => '(CZ)?[0-9]{8,10}',                                  # Czech Republic
        'DE'  => '(DE)?[0-9]{9}',                                     # Germany
        'DK'  => '(DK)?[0-9]{8}',                                     # Denmark
        'EE'  => '(EE)?[0-9]{9}',                                     # Estonia
        'GR'  => '(EL)?[0-9]{9}',                                     # Greece
        'ES'  => '(ES)?[A-Z][0-9]{7}(?:[0-9]|[A-Z])',                 # Spain
        'FI'  => '(FI)?[0-9]{8}',                                     # Finland
        'FR'  => '(FR)?[0-9A-Z]{2}[0-9]{9}',                          # France
        'HR'  => '(HR)?[0-9]{11}',                                    # Croatia
        'HU'  => '(HU)?[0-9]{8}',                                     # Hungary
        'IE'  => '(IE)?([0-9]{7}[A-Z]{1,2}|[0-9][A-Z][0-9]{5}[A-Z])', # Ireland
        'IT'  => '(IT)?[0-9]{11}',                                    # Italy
        'LT'  => '(LT)?([0-9]{9}|[0-9]{12})',                         # Lithuania
        'LU'  => '(LU)?[0-9]{8}',                                     # Luxembourg
        'LV'  => '(LV)?[0-9]{11}',                                    # Latvia
        'MT'  => '(MT)?[0-9]{8}',                                     # Malta
        'NL'  => '(NL)?[0-9]{9}B[0-9]{2}',                            # Netherlands
        'PL'  => '(PL)?[0-9]{10}',                                    # Poland
        'PT'  => '(PT)?[0-9]{9}',                                     # Portugal
        'RO'  => '(RO)?[0-9]{2,10}',                                  # Romania
        'SE'  => '(SE)?[0-9]{12}',                                    # Sweden
        'SI'  => '(SI)?[0-9]{8}',                                     # Slovenia
        'SK'  => '(SK)?[0-9]{10}',                                    # Slovakia
        'XI'  => '(XI)?([0-9]{9}([0-9]{3})?|[A-Z]{2}[0-9]{3})'        # North Ireland
    ];

    public $tax_manager_class;
    public $id_customer;
    public $valid = 0;

    public static $valid_force = 0;
    public static $config = [];

    private static $_hookActionCartSave_returned = false;

    private $errors = [];
    private $vatNumber = '';

    private $SOAPUrl = 'http://ec.europa.eu/taxation_customs/vies/checkVatService.wsdl';

    private $cacheTime = 864000;

    protected $module_config = [
        'VATCHECKER_LIVE_MODE',
        'VATCHECKER_IGNORE_COUNTRY',
        'VATCHECKER_SHOP_VAT',
        'VATCHECKER_CHECK_NAME',
        'VATCHECKER_CHECK_NAME_PERCENT',
        'VATCHECKER_CHECK_NAME_COUNTRIES',
        'VATCHECKER_CHECK_ADDRESS',
        'VATCHECKER_CHECK_ADDRESS_PERCENT',
        'VATCHECKER_CHECK_ADDRESS_COUNTRIES',
        'VATCHECKER_CHECK_ON_CART'
    ];

    public function __construct()
    {
        $this->name = 'vatchecker';
        $this->tab = 'billing_invoicing';
        $this->version = '1.0.1';
        $this->author = 'metacreo';
        $this->need_instance = 0;
        $this->tax_manager_class = 'VatCheckerTaxManager';
        $this->bootstrap = true;
        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];
        parent::__construct();
        $this->displayName = $this->l('Vat Checker');
        $this->description = $this->l('VAT number validation and TAX manipulation module.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
        $this->init();
    }

    private function init()
    {
        if ($this->active) {
            if (function_exists('apcu_fetch')) {
                self::$config = apcu_fetch('vatchecker');
            }
            if (!self::$config) {
                $conf = Configuration::getMultiple($this->module_config);
                foreach ($conf as $k => $v) {
                    self::$config[$k] = $v;
                }
                self::$config['customersForceNoVat'] = $this->getCustomersForceNoVat();
                self::$config['PS_TAX_ADDRESS_TYPE'] = Configuration::get('PS_TAX_ADDRESS_TYPE');
                self::$config['VATCHECKER_NO_TAX_GROUP'] = Configuration::get('VATCHECKER_NO_TAX_GROUP');
                if (function_exists('apcu_store')) {
                    apcu_store('vatchecker', self::$config);
                }
            }
            if (!$this->id_customer) {
                $this->id_customer = isset($this->context->customer->id) ? (int) $this->context->customer->id : 0;
            }
            if ($this->id_customer && in_array($this->id_customer, self::$config['customersForceNoVat'])) {
                self::$valid_force = 1;
            }
        }
    }

    public function install()
    {
        if (extension_loaded('soap') == false) {
            $this->_errors[] = $this->l('You have to enable the SOAP extension on your server to install this module');
            return false;
        }
        Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . $this->name . '` (
                `id` int UNSIGNED NOT NULL AUTO_INCREMENT,
                `id_customer` int NOT NULL UNIQUE,
                `active` tinyint(1) NOT NULL,
                `add_by` int NOT NULL,
                `upd_by` int NOT NULL,
                `date_add` datetime NOT NULL,
                `date_upd` datetime NOT NULL,
                PRIMARY KEY (`id`)
            )
        ');
        Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . $this->name . '_cache` (
                `id_cache` int UNSIGNED NOT NULL AUTO_INCREMENT,
                `countryCode` varchar(2) NOT NULL,
                `vatNumber` varchar(32) NOT NULL,
                `requestDate` datetime NOT NULL DEFAULT "0000-00-00 00:00:00",
                `valid` tinyint(1) NOT NULL,
                `name` text NOT NULL,
                `address` text NOT NULL,
                `error` text NOT NULL,
                PRIMARY KEY (`id_cache`)
            )
        ');
        $id_tab = Tab::getIdFromClassName('AdminVatChecker');
        if (!$id_tab) {
            $tab = new Tab();
            $tab->active = 1;
            $tab->class_name = 'AdminVatChecker';
            $tab->module = $this->name;
            $tab->id_parent = (int)Tab::getIdFromClassName('AdminParentCustomer');
            foreach (Language::getLanguages(false) as $lang) {
                $tab->name[(int) $lang['id_lang']] = $this->l('VAT Excluded');
            }
            if (!$tab->add()) {
                return $this->_abortInstall($this->l('Unable to create the "AdminVatChecker" tab'));
            }
        }
        $lang = Configuration::get('PS_LANG_DEFAULT');
        $group_name = [$lang => $this->l('Customer VAT Free') . ' (' . $this->name . ')'];
        if ($group = Group::searchByName($group_name[$lang])) {
            $id_group = $group['id_group'];
        } else {
            $group = new Group();
            $group->name = $group_name;
            $group->price_display_method = 1;
            $group->date_add = date('Y-m-d H:i:s');
            $group->add(false, false);
            $id_group = $group->id;
        }
        Configuration::updateValue('VATCHECKER_NO_TAX_GROUP', $id_group);
        Configuration::updateValue('VATCHECKER_LIVE_MODE', false);
        Configuration::updateValue('VATCHECKER_IGNORE_COUNTRY', 0);
        Configuration::updateValue('VATCHECKER_SHOP_VAT', null);
        Configuration::updateValue('VATCHECKER_CHECK_NAME', false);
        Configuration::updateValue('VATCHECKER_CHECK_NAME_PERCENT', '50');
        Configuration::updateValue(
            'VATCHECKER_CHECK_NAME_COUNTRIES',
            'AT,BE,BG,CY,CZ,DK,EE,FI,FR,GR,HR,HU,IE,IT,LT,LU,LV,MT,NL,PL,PT,RO,SE,SI,SK'
        );
        Configuration::updateValue('VATCHECKER_CHECK_ADDRESS', false);
        Configuration::updateValue('VATCHECKER_CHECK_ADDRESS_PERCENT', '20');
        Configuration::updateValue(
            'VATCHECKER_CHECK_ADDRESS_COUNTRIES',
            'AT,BE,BG,CY,CZ,DK,EE,FI,FR,GR,HR,HU,IE,IT,LT,LU,LV,MT,NL,PL,PT,RO,SE,SI,SK'
        );
        return parent::install() && $this->registerHook('actionValidateCustomerAddressForm') &&
            $this->registerHook('actionCartSave') && $this->registerHook('actionTaxManager') &&
            $this->registerHook('actionObjectCustomerDeleteBefore');
    }

    public function uninstall()
    {
        $group = new Group(Configuration::get('VATCHECKER_NO_TAX_GROUP'));
        $group->delete();
        foreach ($this->module_config as $c) {
            Configuration::deleteByName($c);
        }
        Configuration::deleteByName('VATCHECKER_NO_TAX_GROUP');
        Db::getInstance()->execute('DROP TABLE IF EXISTS `' . _DB_PREFIX_ . $this->name . '`');
        Db::getInstance()->execute('DROP TABLE IF EXISTS `' . _DB_PREFIX_ . $this->name . '_cache`');
        if ($id_tab = Tab::getIdFromClassName('AdminVatChecker')) {
            $tab = new Tab((int)$id_tab);
            $tab->delete();
        }
        return parent::uninstall();
    }

    public function disable($force_all = false)
    {
        Configuration::updateValue('VATCHECKER_LIVE_MODE', false);
        return parent::disable($force_all);
    }

    public function enable($force_all = false)
    {
        Configuration::updateValue('VATCHECKER_LIVE_MODE', true);
        return parent::enable($force_all);
    }

    public function getContent()
    {
        $this->context->smarty->assign('module_dir', $this->_path);
        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');
        if ((Tools::isSubmit('submit'.$this->name)) == true) {
            foreach ($this->module_config as $v) {
                Configuration::updateValue($v, Tools::getValue($v));
            }
            $output .= $this->displayConfirmation($this->l('Settings updated'));
            if (!Configuration::get('VATCHECKER_LIVE_MODE')) {
                if ($vat_number = Configuration::get('VATCHECKER_SHOP_VAT')) {
                    $test_vat = $this->getViesRemote(preg_replace('/\W/', '', strtoupper($vat_number)));
                    if (isset($test_vat->response)) {
                        $response = [$this->l('VIES service response OK. You can switch to Live mode.')];
                        foreach ($test_vat->response as $k => $r) {
                            $response[] = strtoupper(implode(' ', preg_split('/(?=[A-Z])/', $k))) . ': ' . $r;
                        }
                        $output .= $this->displayConfirmation(implode('<br>', $response));
                    }
                    if (isset($test_vat->error)) {
                        $output .= $this->displayError($test_vat->error);
                    }
                } else {
                    $output .= $this->displayError($this->l('Please enter Shop VAT Number to make test'));
                }
            }
            if (function_exists('apcu_delete')) {
                apcu_delete('vatchecker');
            }
        }
        $output .= $this->renderForm();
        return $output;
    }

    protected function renderForm()
    {
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
        $countries = Country::getCountries($default_lang);
        $eu_countries = array_keys($this->euVatCountriesAndFormats);
        $countries_list = [
            0 => [
                'id' => 0,
                'name' => $this->l('- Select a country -')
            ]
        ];
        foreach ($countries as $country) {
            if (in_array($country['iso_code'], $eu_countries)) {
                $countries_list[] = [
                    'id' => $country['id_country'],
                    'name' => $country['name']
                ];
            }
        }
        $intlFeature = (extension_loaded('intl') == false) ? true : false;
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs'
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Module Live mode'),
                        'name' => 'VATCHECKER_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Set NO and click Save to test VIES service.'),
                        'values' => [
                            [
                                'id' => 'live_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ],
                            [
                                'id' => 'live_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            ]
                        ]
                    ],
                    [
                        'col' => 6,
                        'type' => 'text',
                        'label' => $this->l('Shop VAT Number'),
                        'name' => 'VATCHECKER_SHOP_VAT',
                        'desc' => $this->l('Your VAT number with country ISO prefix. ( LLNNNNNNN... )')
                    ],
                    [
                        'col' => 3,
                        'type' => 'select',
                        'desc' => $this->l('Select country to ignore. Shop origin country.'),
                        'name' => 'VATCHECKER_IGNORE_COUNTRY',
                        'label' => $this->l('Ignore origin country'),
                        'options' => [
                            'query' => $countries_list,
                            'id' => 'id',
                            'name' => 'name'
                        ]
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Check company name'),
                        'name' => 'VATCHECKER_CHECK_NAME',
                        'is_bool' => true,
                        'desc' => $this->l('Compares customer entered company name with VIES data.'),
                        'values' => [
                            [
                                'id' => 'name_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ],
                            [
                                'id' => 'name_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            ]
                        ],
                        'disabled' => $intlFeature
                    ],
                    [
                        'col' => 1,
                        'type' => 'text',
                        'label' => $this->l('Name match percent'),
                        'name' => 'VATCHECKER_CHECK_NAME_PERCENT',
                        'desc' => $this->l('Percent of match name entered and given from VIES.'),
                        'disabled' => $intlFeature
                    ],
                    [
                        'col' => 8,
                        'type' => 'text',
                        'label' => $this->l('Check company name EU Countries'),
                        'name' => 'VATCHECKER_CHECK_NAME_COUNTRIES',
                        'desc' => $this->l('Comma separated EU countries ISO 2 digits codes.') .
                        ' <span style="color:red;">(' . $this->l('This feature may not works with DE and ES.') . ')<span>',
                        'disabled' => $intlFeature
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Check company address'),
                        'name' => 'VATCHECKER_CHECK_ADDRESS',
                        'is_bool' => true,
                        'desc' => $this->l('Compares customer entered company address with VIES data.'),
                        'values' => [
                            [
                                'id' => 'address_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ],
                            [
                                'id' => 'address_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            ]
                        ],
                        'disabled' => $intlFeature
                    ],
                    [
                        'col' => 1,
                        'type' => 'text',
                        'label' => $this->l('Address match percent'),
                        'name' => 'VATCHECKER_CHECK_ADDRESS_PERCENT',
                        'desc' => $this->l('Percent of match address entered and given from VIES.'),
                        'disabled' => $intlFeature
                    ],
                    [
                        'col' => 8,
                        'type' => 'text',
                        'label' => $this->l('Check company address EU Countries'),
                        'name' => 'VATCHECKER_CHECK_ADDRESS_COUNTRIES',
                        'desc' => $this->l('Comma separated EU countries ISO 2 digits codes.').
                        ' <span style="color:red;">(' . $this->l('This feature may not works with DE and ES.') . ')<span>',
                        'disabled' => $intlFeature
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Check on cart save'),
                        'name' => 'VATCHECKER_CHECK_ON_CART',
                        'is_bool' => true,
                        'desc' => $this->l('Check VIES data on every cart save or address change.'),
                        'values' => [
                            [
                                'id' => 'cart_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ],
                            [
                                'id' => 'cart_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            ]
                        ]
                    ]
                ],
                'submit' => [
                    'title' => $this->l('Save')
                ]
            ]
        ];
        $helper = new HelperForm();
        $helper->module = $this;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
        $helper->title = $this->displayName;
        $helper->submit_action = 'submit'.$this->name;
        foreach ($this->module_config as $v) {
            $helper->fields_value[$v] = Configuration::get($v);
        }
        return $helper->generateForm([$fields_form]);
    }

    private function getConfig($x)
    {
        switch ($x) {
            case 'name':
                $c = 'VATCHECKER_CHECK_NAME_COUNTRIES';
                break;
            case 'address':
                $c = 'VATCHECKER_CHECK_ADDRESS_COUNTRIES';
                break;
        }
        $return = [];
        $members = explode(',', self::$config[$c]);
        if (is_array($members)) {
            foreach ($members as $member) {
                if (preg_match('/[A-Z]{2}/i', $member)) {
                    $return[] = strtoupper($member);
                }
            }
        }
        return $return;
    }

    private function getVies($vatNumber)
    {
        if (substr($vatNumber, 0, 2) == 'GR') {
            $vatNumber = 'EL' . preg_replace('/\D/', '', $vatNumber);
        }
        if ($cached = $this->getViesCached($vatNumber)) {
            return $cached;
        } else {
            return $this->insertViesCached($vatNumber);
        }
        return false;
    }

    private function getViesCached($vatNumber)
    {
        $sql = 'SELECT * FROM `' . _DB_PREFIX_ . $this->name . '_cache` WHERE ';
        $sql .= '`vatNumber` = "' . $vatNumber . '"';
        $cached = Db::getInstance()->getRow($sql);
        if (isset($cached['requestDate'])) {
            if ((time() - strtotime($cached['requestDate'])) > $this->cacheTime) {
                $this->deleteViesCached($cached['id_cache']);
                return false;
            }
        }
        return $cached;
    }

    private function insertViesCached($vatNumber)
    {
        $vies = $this->getViesRemote($vatNumber);
        $sql = [
            'countryCode' => $vies->response->countryCode ?? null,
            'vatNumber' => $vatNumber,
            'requestDate' => date('Y-m-d H:i:s'),
            'valid' => $this->valid = ($vies->response->valid ?? 0) ? 1 : 0,
            'name' => ($vies->response->name ?? null) ? pSQL($vies->response->name) : null,
            'address' => ($vies->response->address ?? null) ? pSQL($vies->response->address) : null,
            'error' => ($vies->error ?? null) ? pSQL($vies->error) : null
        ];
        if (Db::getInstance()->insert($this->name . '_cache', [$sql])) {
            return $sql;
        }
    }

    private function deleteViesCached($id_cache)
    {
        return Db::getInstance()->execute(
            'DELETE FROM `' . _DB_PREFIX_ . $this->name . '_cache` WHERE `id_cache` = ' . (int) $id_cache . ' OR
            STR_TO_DATE(`requestDate`, "%Y-%m-%d%H:%i:%s") < (NOW() - INTERVAL ' . (int) $this->cacheTime . ' SECOND)'
        );
    }

    private function getViesRemote($vatNumber)
    {
        $return = new stdClass();
        try {
            $client = new SoapClient($this->SOAPUrl);
            if (!$client) {
                return $return->error = 'webservice at ec.europa.eu unavailable';
            }
            $return->response = $client->checkVat(
                [
                    'countryCode' => substr($vatNumber, 0, 2),
                    'vatNumber' => substr($vatNumber, 2)
                ]
            );
        } catch (SoapFault $e) {
            $faults = [
                'INVALID_INPUT' => 'The provided CountryCode is invalid or the VAT number is empty.',
                'SERVICE_UNAVAILABLE' => 'The SOAP service is unavailable, try again later.',
                'MS_UNAVAILABLE' => 'The Member State service is unavailable, try again later.',
                'TIMEOUT' => 'The Member State service could not be reached in time, try again later.',
                'SERVER_BUSY' => 'The service cannot process your request. Try again later.'
            ];
            $return->error = $faults[$e->faultstring] ?? $e->faultstring;
        }
        return $return;
    }

    private function getCustomersForceNoVat()
    {
        if ($query = Db::getInstance()->executeS(
            'SELECT `id_customer` FROM `' . _DB_PREFIX_ . $this->name . '` WHERE `active` = 1'
        )) {
            foreach ($query as $q) {
                $return[] = $q['id_customer'];
            }
        }
        return $return ?? [];
    }

    private function manageVatGroup()
    {
        if (self::$valid_force || !$this->id_customer) {
            return;
        }
        $customer_groups = Customer::getGroupsStatic($this->id_customer);
        $id_tax_free_group = (int) self::$config['VATCHECKER_NO_TAX_GROUP'];
        $in_tax_free_group = in_array($id_tax_free_group, $customer_groups);
        if ($this->valid && !$in_tax_free_group) {
            $this->context->customer->addGroups([$id_tax_free_group]);
        }
        if (!$this->valid && $in_tax_free_group) {
            if ($key = array_search($id_tax_free_group, $customer_groups)) {
                unset($customer_groups[$key]);
            }
            $this->context->customer->updateGroup($customer_groups);
        }
    }

    private function checkData($vatNumber, $countryIso, $address)
    {
        $ret = [];
        $vat = $countryIso . preg_replace('/\W/', '', str_replace($countryIso, null, $vatNumber));
        if ($vies = $this->getVies($vat)) {
            if (isset($vies['error']) && $vies['error']) {
                $this->errors['vat_number'] = $vies['error'];
            } else {
                $this->valid = $vies['valid'] ?? 0;
                $this->vatNumber = $vat;
                if (!$this->valid) {
                    $this->errors['vat_number'] = $this->l('This is not a valid VAT number.');
                } else {
                    if (self::$config['VATCHECKER_CHECK_NAME']) {
                        $this->checkName($address->company, $countryIso, $vies['name'] ?? null);
                    }
                    if (self::$config['VATCHECKER_CHECK_ADDRESS']) {
                        $this->checkAddress($address, $countryIso, $vies['address'] ?? null);
                    }
                }
                if (!$this->errors) {
                    return $this->valid;
                }
            }
        }
    }

    private function checkName($company, $countryIso, $viesName)
    {
        if (in_array($countryIso, $this->getConfig('name'))) {
            $cName = preg_replace( '/\W/u', '', strtolower($this->transLit($company)));
            if ($vName = preg_replace('/\W/u', '', strtolower($this->transLit($viesName)))) {
                similar_text($cName, $vName, $percent);
                if ($percent < self::$config['VATCHECKER_CHECK_NAME_PERCENT']) {
                    $this->errors['company'] = $this->l('Company not match VIES data:') . '<br>' .
                        $viesName;
                }
            } else {
                $this->errors['company'] = $this->l('Cannot verify your company name.');
            }
        }
    }

    private function checkAddress($addr, $countryIso, $viesAddress)
    {
        if (in_array($countryIso, $this->getConfig('address'))) {
            $cAddr = preg_replace('/\W/u', '', strtolower($this->transLit($addr->address1 . $addr->address2 . $addr->city)));
            if ($vAddr = preg_replace('/\W/u', '', strtolower($this->transLit($viesAddress)))) {
                similar_text($cAddr, $vAddr, $percent);
                if ($percent < (int) self::$config['VATCHECKER_CHECK_ADDRESS_PERCENT']) {
                    $this->errors['address1'] = $this->l('Address not match VIES data:') . '<br>' .
                    $viesAddress;
                }
            } else {
                $this->errors['address1'] = $this->l('Cannot verify your company address.');
            }
        }
    }

    public function hookActionValidateCustomerAddressForm($params)
    {
        if (self::$config['VATCHECKER_LIVE_MODE']) {
            if (self::$valid_force) {
                return;
            }
            $form = $params['form'];
            $countryId = $form->getField('id_country')->getValue();
            $countryIso = Country::getIsoById($countryId);
            if (self::$config['VATCHECKER_IGNORE_COUNTRY'] != $countryId) {
                if (in_array($countryIso, array_keys($this->euVatCountriesAndFormats))) {
                    $vatNumber = $form->getField('vat_number')->getValue();
                    if (strlen($vatNumber) > 0) {
                        if ($this->checkVatFormat($vatNumber)) {
                            $address = new Address();
                            $address->company = $form->getField('company')->getValue();
                            $address->address1 = $form->getField('address1')->getValue();
                            $address->address2 = $form->getField('address2')->getValue();
                            $address->city = $form->getField('city')->getValue();
                            if ($this->checkData($vatNumber, $countryIso, $address)) {
                                $form->getField('vat_number')->setValue($this->vatNumber);
                            }
                        } else {
                            $this->errors['vat_number'] = $this->l('VAT number invalid format.');
                        }
                    }
                    if ($this->errors) {
                        foreach ($this->errors as $k => $v) {
                            $form->getField($k)->addError($v);
                        }
                    }
                }
            }
            return $this->manageVatGroup();
        }
    }

    public function hookActionCartSave($params)
    {
        if (!self::$_hookActionCartSave_returned) {
            self::$_hookActionCartSave_returned = true;
            if (self::$config['VATCHECKER_LIVE_MODE'] && self::$config['VATCHECKER_CHECK_ON_CART']) {
                if (isset($params['cart']->id_customer)) {
                    if (self::$valid_force || !$params['cart']->id_customer) {
                        return;
                    }
                    $ps_tax_address_type = self::$config['PS_TAX_ADDRESS_TYPE'];
                    $address = new Address((int) $params['cart']->$ps_tax_address_type);
                    $vatNumber = preg_replace('/[^A-Z0-9]/', '', strtoupper($address->vat_number));
                    if (empty($vatNumber)) {
                        $this->valid = 0;
                    } else {
                        $countryIso = Country::getIsoById($address->id_country);
                        if (Country::getIsoById(self::$config['VATCHECKER_IGNORE_COUNTRY']) != $countryIso) {
                            if (in_array($countryIso, array_keys($this->euVatCountriesAndFormats))) {
                                if (strlen($vatNumber) > 0) {
                                    $this->checkData($vatNumber, $countryIso, $address);
                                }
                            }
                        }
                    }
                    return $this->manageVatGroup();
                }
            }
        }
    }

    public function hookActionObjectCustomerDeleteBefore($params)
    {
        if ($id_customer = $params['object']->id ?? 0) {
            return Db::getInstance()->execute(
                'DELETE FROM `' . _DB_PREFIX_ . $this->name . '` WHERE `id_customer` = ' . (int) $id_customer
            );
        }
    }

    private function checkVatFormat($vatNumber)
    {
        if (!$vatNumber) {
            return false;
        }
        $formats = implode( '|', $this->euVatCountriesAndFormats);
        return preg_match('/(?xi)^(' . $formats . ')$/', $vatNumber);
    }

    private function transLit($string)
    {
        return transliterator_transliterate(
            "Any-Latin; NFD; [:Nonspacing Mark:] Remove; NFC; [:Punctuation:] Remove; Lower();",
            $string
        );
    }
}
